// A set of useful vector functions.
// Written by Nils Liaaen Corneliusen 2016.
// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license
// https://www.ignorantus.com/

#ifndef VECH
#define VECH

typedef struct {
    float x, y;
} vec2;

static inline vec2 vec2_sub( vec2 a, vec2 b )
{
    vec2 v;
    v.x = a.x - b.x;
    v.y = a.y - b.y;
    return v;
}

static inline vec2 vec2_set( float x, float y )
{
    vec2 v;
    v.x = x;
    v.y = y;
    return v;
}

static inline float vec2_dot( vec2 a, vec2 b )
{
    return a.x*b.x + a.y*b.y;
}


typedef struct {
    float x, y, z;
} vec3;

static inline vec3 vec3_set( float x, float y, float z )
{
    vec3 v;
    v.x = x;
    v.y = y;
    v.z = z;
    return v;
}

static inline vec3 vec3_sub( vec3 a, vec3 b )
{
    vec3 v;
    v.x = a.x - b.x;
    v.y = a.y - b.y;
    v.z = a.z - b.z;
    return v;
}

static inline vec3 vec3_add( vec3 a, vec3 b )
{
    vec3 v;
    v.x = a.x + b.x;
    v.y = a.y + b.y;
    v.z = a.z + b.z;
    return v;
}

static inline vec3 vec3_mul( vec3 a, vec3 b )
{
    vec3 v;
    v.x = a.x * b.x;
    v.y = a.y * b.y;
    v.z = a.z * b.z;
    return v;
}

static inline vec3 vec3_mul1( vec3 a, float b )
{
    vec3 v;
    v.x = a.x * b;
    v.y = a.y * b;
    v.z = a.z * b;
    return v;
}

static inline vec3 vec3_fmin1( vec3 a, float f )
{
    vec3 v;
    v.x = a.x < f ? a.x : f;
    v.y = a.y < f ? a.y : f;
    v.z = a.z < f ? a.z : f;
    return v;
}

static inline float vec3_dot( vec3 a, vec3 b )
{
    return a.x*b.x + a.y*b.y + a.z*b.z;
}

static inline vec3 vec3_normalize( vec3 a )
{
    return vec3_mul1( a, 1.0f / sqrtf( vec3_dot( a, a ) ) );
}

static inline vec3 vec3_mix( vec3 x, vec3 y, float a )
{
    return vec3_add( vec3_mul1( x, 1.0f-a ), vec3_mul1( y, a ) );
}

static inline vec3 vec3_reflect( vec3 i, vec3 n )
{
    return vec3_sub( i, vec3_mul1( n, vec3_dot( n, i ) * 2.0f ) );
}

#endif
